import 'dart:async';

import 'package:kohana/kohana.dart';
import 'package:flutter/material.dart';
import 'base_widget_options.dart';
import 'package:rxdart/rxdart.dart';

class BaseInputFields extends BaseViewModel {
  final BaseInputWidgetOptions options;
  final validationMessageSubject = BehaviorSubject.seeded("");
  final changesSubject = BehaviorSubject<String>();

  String get validationMessage => validationMessageSubject.value!;
  final TextEditingController controller;

  String get value => controller.text;

  Stream<String> get changes => changesSubject.stream;

  Stream<String> asStream() => validationMessageSubject.stream;

  BaseInputFields._(this.options, this.controller);

  factory BaseInputFields(BaseInputWidgetOptions options) {
    var controller = options.controller ?? TextEditingController(text: options.value);
    return BaseInputFields._(options, controller);
  }

  void setValidationMessage(String? message) {
    validationMessageSubject.add(message ?? "");
  }

  @override
  FutureOr start() {
    var focusNode = options.focusNode;

    controller.addListener(() {
      changesSubject.add(controller.text);
    });

    focusNode?.addListener(() {
      if (focusNode.hasFocus) {
        validationMessageSubject.add("");
      }
    });

    var inputValueStream = options.inputValueStream;

    if (inputValueStream != null) {
      disposeBag.addAll([
        inputValueStream.listen(
          (event) {
            if (event.value != value) {
              controller.text = event.value;
            }

            validationMessageSubject.add((event.value).isEmpty ? '' : event.validationMessage ?? "");
          },
        )
      ]);
    }

    var listener = options.changeListener;
    if (listener != null) {
      disposeBag.addAll([
        changes.listen((event) {
          listener(InputValue(event, fromInput: true, validationMessage: validationMessage));
        }),
      ]);
    }

    var watcher = options.watcher;
    if (watcher != null) {
      print("Ima watchera");
      disposeBag.addAll([
        watcher(this),
      ]);
    }

    var onMounted = options.onMounted;
    if (onMounted != null) {
      onMounted(this);
    }
  }

  @override
  FutureOr dispose() {
    validationMessageSubject.close();
    changesSubject.close();
    controller.dispose();
    options.focusNode?.dispose();
    super.dispose();
  }
}
