import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:kohana/kohana.dart';

import 'base_input_fields.dart';

typedef InputDecoration InputDecorationBuilder(InputDecoration inputDecoration, BaseInputFields $);
typedef TextField TextFieldBuilder(TextField textField, BaseInputFields $);
typedef TextStyle TextStyleBuilder(TextStyle textStyle, BaseInputFields $);
typedef BaseInputWidgetStateWatcher = StreamSubscription Function(BaseInputFields);
typedef BaseInputWidgetOnMounted = void Function(BaseInputFields);

class BaseInputWidgetOptions {
  final InputDecorationBuilder? inputDecorationBuilder;
  final TextFieldBuilder? textFieldBuilder;
  final TextStyleBuilder? textStyleBuilder;
  final Stream<InputValue>? inputValueStream;
  final Consumer<InputValue>? changeListener;
  final BaseInputWidgetStateWatcher? watcher;
  final BaseInputWidgetOnMounted? onMounted;

  final TextEditingController? controller;
  final String? value;
  final List<TextInputFormatter>? inputFormatters;
  final FocusNode? nextFocusNode;
  final FocusNode? focusNode;
  final bool? autofocus;
  final TextAlign textAlign;
  final bool showErrorLabel;

  final String? label;
  final String? placeHolder;
  final TextInputType? type;
  final bool obscureText;
  final bool readOnly;
  final bool enabled;
  final TextStyle? textStyle;

  const BaseInputWidgetOptions({
    this.inputDecorationBuilder,
    this.textFieldBuilder,
    this.textStyleBuilder,
    this.focusNode,
    this.nextFocusNode,
    this.inputValueStream,
    this.changeListener,
    this.watcher,
    this.onMounted,
    this.controller,
    this.value,
    this.inputFormatters,
    this.autofocus,
    this.textAlign = TextAlign.start,
    this.label,
    this.placeHolder,
    this.type,
    this.obscureText = false,
    this.readOnly = false,
    this.showErrorLabel = true,
    this.enabled = true,
    this.textStyle,
  });

  @override
  String toString() {
    return 'BaseInputWidgetOptions{inputDecorationBuilder: $inputDecorationBuilder, textFieldBuilder: $textFieldBuilder, focusNode: $focusNode, nextFocusNode: $nextFocusNode, inputValueStream: $inputValueStream, changeListener: $changeListener, controller: $controller}';
  }
}
